from plugin_types import *
import os

@plugin
class Sitemap(object):
    """ All pages need to register with this (via addNode) to be included
        in a the sitemap. The sitemap is rendered grouped by section.
    """

    #@plugin - class is plugin, which means __call__ is used
    def __call__(self, page):
        """ Indicate sitemap is to be rendered. Save page to re-render later.
        """
        if self.finalize:
            return self._render()
        elif page not in self.contexts:
            self.contexts.append(page)
            return 'Rendering delayed. This should be replaced.'

    @post_site_hook
    def renderSitemap(self, tree):
        """ Post_site_hook to render sitemap in registered contexts.
        Works by re-publishing the context/pages.
        """
        self.finalize = True
        for page in self.contexts:
            page.publish()

    @post_page_hook
    def ping(self, page):
        """ Adds children pages to sitemap. Done here so as to allow
            for appropriate ordering.
            Also use nav_order so the sitemap corresponds to the top nav.
        """
        if page.hidden() or page.nonav(): return
        if not ( 'index_html' in page.filenames or
                 'index.html' in page.filenames ): return
        sitemap = self.sitemap
        path = page.site_path()
        parent_path = os.path.dirname(path)
        if path != parent_path:
            sitemap.setdefault(parent_path, []).append(page)

    ####################################################################
    ### internal 
    def __init__(self):
        self.sitemap = {}
        self.contexts = []
        self.finalize = False

    def _render(self, section='/'):
        results = []
        self._ul_list(self.sitemap[section], 1, results)
        return ''.join(results)

    def _ul_list(self, section, depth, results):
        results.append("  "*depth)
        if depth > 1: results.append('<li style="list-style-type:none">')
        results.append("<ul>\n")

        sitemap = self.sitemap
        section.sort(key=lambda p:str(p.get('nav_order', p.dirpath)))
        for page in section:
            results.append("  "*(depth))
            results.append(('<li>'
                    '<a href="%s" title="Last Modified: %s">%s</a></li>\n')
                    %(page.base_path(), page.last_modified(), page.title()))
            if sitemap.has_key(page.site_path()): # sub-section list
                self._ul_list(sitemap[page.site_path()], depth+1, results)
        results.append("  "*(depth))
        results.append("</ul>\n")
        if depth > 1: results.append("</li>")

