#!/usr/bin/env python

import os
from PageTemplate import PageTemplate
from options import options

def template_tree(options):
    """ Loop through site tree creating PageTemplate instances for each dir.
    Also build sitemap.
    """
    torender = []
    (parents, lvl, page) = ({0: None}, 0, None)
    for (dirpath, dirnames, filenames) in os.walk(options.root):
        slashes = dirpath.count('/')
        if lvl < slashes:
            parents[slashes] = page
        parent = parents[slashes]
        lvl = slashes

        page = PageTemplate(options, dirpath, dirnames, filenames, parent)
        torender.append(page)

    return torender

def run():
    (pre_site, pre_page, post_page, post_site) = options.plugins.getHooks()
    tree = template_tree(options)
    for ps in pre_site: ps(tree)
    for page in tree:
        if not page.hidden():
            for pp in pre_page: pp(page)
            page.publish()
            for pp in post_page: pp(page)
    for ps in post_site: ps(tree)

if __name__=='__main__':
    run()
#    import profile
#    profile.run('run()')


